@@ -22,6 +22,7 @@ SOCKETRETRIES = 3
 SOCKETWAITTIME = 1
 KEEPALIVETIMEOUT = 25
 RECONNECTTIMEOUT = 5
+TIDBLOCKTIME = 90
 
 class DcsConnection:
     __serial__ = None
@@ -268,15 +269,18 @@ class DcsConnection:
 
         if(msgtype == 6):
             logging.info(str(id(self)) + " TID: " + str(tid) + "; Calc Log Daten (für MSG.Type 2).")
+            nack=True
             try:
                 self.loadlogdata()
+                nack=False
             except Exception as e:
                 logging.warning(str(id(self)) + " TID: " + str(tid) + "; Error getting logData")
             try:
-                self.__send__(None, 9, tid, isresponce=True)     #send ACK
+                self.__send__(None, 9, tid, isresponce=True, nack=nack)     #send ACK/NACK
             except Exception as e:
-                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending ACK for Calc Log Data")
-            self.clearlogdata()
+                logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending ACK/NACK for Calc Log Data. NACK=" + str(nack))
+            if nack == False:
+                self.clearlogdata()
             return
 
         if(msgtype == 7):
@@ -285,6 +289,10 @@ class DcsConnection:
                 self.sendlogdata(tid=tid)
             except Exception as e:
                 logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending logData.")
+                try:
+                    self.__send__(None, 9, tid, isresponce=True, nack=True)     #send NACK
+                except Exception as sendexception:
+                    logging.error(str(id(self)) + " TID: " + str(tid) + "; Error sending NACK for Send Log Data. " + str(sendexception)) 
             return
 
         logging.info(str(id(self)) + " TID: " + str(tid) + "; Unknown MSG-Type received: " + str(msgtype) + ".")
@@ -334,7 +342,7 @@ class DcsConnection:
         cleanList = []
         for request in self.__sentRequests__:
             try:
-                if self.__sentRequests__[request] != None and self.__sentRequests__[request]["timestamp"] < datetime.now() - timedelta(seconds=90):
+                if self.__sentRequests__[request] != None and self.__sentRequests__[request]["timestamp"] < datetime.now() - timedelta(seconds=TIDBLOCKTIME):
                     cleanList.append(request)
             except Exception as e:
                 logging.debug(str(id(self)) + " request " + str(request) + " element does not exist anymore. " + str(e))
