@@ -5,7 +5,6 @@ from datetime import datetime, timedelta
 import logging
 import time
 import json
-
 import sys
 import os
 
@@ -21,16 +20,18 @@ REGISTERTIMEOUT = 5
 BUILDNR = "4321"
 
 class DeviceDcElwa(ModbusRtuDevice):
-    __firmwareVersion__ = "1." + str(BUILDNR)
     __devicetype__ = "ELWA"
+    __irerror__ = False
 
     def __init__(self, serial, nodeid):
         ModbusRtuDevice.__init__(self, serial, nodeid)
+        self.__startRegister__ = 1000
+        self.__registersToRead__ = 24
 
     def __createsetup__(self):
         return {
             "device": self.__devicetype__, \
-            "fwversion": self.__firmwareVersion__, \
+            "fwversion": self.__firmwareversion__, \
             "serialno": self.__serial__, \
             "elno": self.__nodeId__, \
             "ww2target": None, \
@@ -76,9 +77,14 @@ class DeviceDcElwa(ModbusRtuDevice):
             power = dcPower
             meter = None
 
+        errorcode = None
+
+        if self.__irerror__:
+            errorcode = 1
+
         data={
             "device": self.__devicetype__,
-            "fwversion": self.__firmwareVersion__,
+            "fwversion": self.__firmwareversion__,
             "loctime": time.strftime("%H:%M:%S"),
             "dev_id": self.__nodeId__,
             "day_counter" : self.getdataset("day_counter"),
@@ -108,10 +114,17 @@ class DeviceDcElwa(ModbusRtuDevice):
             "temp2": self.getdataset("temp2"),
             "ww2target": self.getdataset("boost_temp_target"),
             "ww2offset": self.getdataset("ww2offset_calibration"),
-            "modbuserrorrate":  self.getcommunicationerrorsrate()
-            }            
+            "modbuserrorrate":  self.getcommunicationerrorsrate(),
+            "errorcode": errorcode
+        }       
         return data
 
+    def getdataset(self, datasetname):
+        if self.__irerror__ == False:
+            return ModbusRtuDevice.getdataset(self, datasetname)
+        else:
+            return None
+
     def getlogdata(self):
         acRelayState = self.getlogvalue("ac_relay_state")
         if acRelayState != None:
@@ -176,6 +189,18 @@ class DeviceDcElwa(ModbusRtuDevice):
         self.__logdata__.clear()
         return logData
 
+    def getintlogvalue(self, datasetname):
+        if self.__irerror__ == False:
+            return ModbusRtuDevice.getintlogvalue(self, datasetname)
+        else:
+            return None
+
+    def getlogvalue(self, datasetname):
+        if self.__irerror__ == False:
+            return ModbusRtuDevice.getlogvalue(self, datasetname)
+        else:
+            return None
+
     def __checkRegiterTimeStamp__(self):
         if self.__registerLastSuccessfullReadTimeStamp__ != None:
             difference = time.time() - self.__registerLastSuccessfullReadTimeStamp__
@@ -186,29 +211,24 @@ class DeviceDcElwa(ModbusRtuDevice):
             return False
     
     def __supervise__(self):
+
+        ModbusRtuDevice.__supervise__(self)
         
-        value = self.getdataset("temp1")
+        value = ModbusRtuDevice.getdataset(self,"temp1")
         if value != None and ( value > 1200 or value < 0 ):
             logging.error("[Device] " + self.getidentifier() + " - Temp1 sensor error: " + str(value))
             temp1error = True
         else:
             temp1error = False
 
-        value = self.getdataset("temp2")
+        value = ModbusRtuDevice.getdataset(self,"temp2")
         if value != None and ( value > 1200 or value < 0 ):
             logging.error("[Device] " + self.getidentifier() + " - Temp2 sensor error: " + str(value))
             temp2error = True
         else:
             temp2error = False
 
-        value = self.getdataset("tempchip")
-        if value == 0 :
-            logging.error("[Device] " + self.getidentifier() + " - No IR Connection to Device.")
-            irError = True
-        else:
-            irError = False
-
-        value = self.getdataset("operation_mode")
+        value = ModbusRtuDevice.getdataset(self,"operation_mode")
         if value != None and value > 100 :
             logging.error("[Device] " + self.getidentifier() + " - Operation Mode " + str(value) + " detected.")
             opmodeError = True
@@ -227,8 +247,17 @@ class DeviceDcElwa(ModbusRtuDevice):
         else:
             modbusError = False
 
-        if self.__checkRegiterTimeStamp__() != True:
-            logging.error("[Device] " + self.getidentifier() + " - Register values to old. Communication errors expected.")
+        value = ModbusRtuDevice.getdataset(self, "tempchip")
+        if value == 0 and modbusError == False:
+            logging.error("[Device] " + self.getidentifier() + " - No IR Connection to Device.")
+            irError = True
+            self.__irerror__ = True
+        else:
+            irError = False
+            self.__irerror__ = False
+
+        if self.__checkRegiterTimeStamp__() != True and modbusError == False:
+            logging.error("[Device] " + self.getidentifier() + " - Register values too old. Communication errors expected.")
             registerError = True
         else:
             registerError = False
@@ -284,11 +313,11 @@ if __name__ == "__main__":
     logging.basicConfig(format='%(asctime)s %(levelname)s [%(threadName)s]: %(message)s', level=logging.INFO)
 
     serial = "120100200505tes1"
-    cryptoKey = "ABABABABABABABABABABABABABABABAB"
+    cryptoKey = "41424142414241424142414241424142"
     serial2 = "120100200505tes2"
-    cryptoKey2 = "ABABABABABABABABABABABABABABABAB"
+    cryptoKey2 = "41424142414241424142414241424142"
     serial3 = "120100200505tes3"
-    cryptoKey3 = "ABABABABABABABABABABABABABABABAB"
+    cryptoKey3 = "41424142414241424142414241424142"
     server = "my-pv.live"
 
     # try to read from valid device
@@ -371,6 +400,26 @@ if __name__ == "__main__":
     else:
         print(color('ERROR: Setting setup value.', fore='red', style='bright'))
 
+    if(device != None):
+        try:
+            register = device.readregister(1021)
+            if register > 0 and register < 900: 
+                print(color('SUCCESS: reading register.', fore='green', style='bright'))
+            else:
+                print(color('ERROR: reading register. Value missmatch: ' + str(register), fore='red', style='bright'))
+        except Exception as e:
+            print(color('ERROR: reading register. Error: ' + str(e), fore='red', style='bright'))
+
+    if(device != None):
+        try:
+            registers = device.readregisters(1000, 10)
+            if registers[1000] !=  0 and registers[1001] != 0:
+                print(color('SUCCESS: reading registers.', fore='green', style='bright'))
+            else:
+                print(color('ERROR: reading registers. Value missmatch: ' + str(registers), fore='red', style='bright'))
+        except Exception as e:
+            print(color('ERROR: reading registers. Error: ' + str(e), fore='red', style='bright'))
+
 
     input("Press ENTER to start running tests")
     logging.getLogger().setLevel(logging.INFO)
