@@ -6,7 +6,7 @@ import logging
 import time
 from colr import color
 
-BUSRECOVERYTIME = 0
+BUSRECOVERYTIME = 0.5
 
 class ModbusConnection:     #Singleton
     __instance__ = None
@@ -33,7 +33,7 @@ class ModbusConnection:     #Singleton
 
         self.__connectionErrorCounter__ = dict()
         self.__connectionCounter__ = dict()
-        self.__modbusClient__ = ModbusClient(method = "rtu", port="/dev/ttyUSB0", stopbits = 1, bytesize = 8, parity = 'N', baudrate= 9600, timeout=0.5, retry_on_empty=True, retry_on_invalid=True)
+        self.__modbusClient__ = ModbusClient(method = "rtu", port="/dev/ttyUSB0", stopbits = 1, bytesize = 8, parity = 'N', baudrate= 9600, timeout=0.2, retry_on_empty=True, retry_on_invalid=True)
 
     def __increaseModbusErrorCounter__(self, nodeId):
         try:
@@ -52,16 +52,16 @@ class ModbusConnection:     #Singleton
             except:
                 self.__connectionErrorCounter__[nodeId] = 0
             counter = self.__connectionErrorCounter__[nodeId]
-        return counter
+            return counter
 
     def getModbusErrorRate(self, nodeId):
         with self.__mutex__:
-            errorRate = 0
+            errorRate = None
             try:
-                errorRate = self.__connectionErrorCounter__[nodeId] / self.__connectionCounter__[nodeId]
+                errorRate = round(self.__connectionErrorCounter__[nodeId] / self.__connectionCounter__[nodeId], 2)
             except:
                 logging.debug("[ModbusConnection] Node " + str(nodeId) + ": Cannot calculate error rate.")
-        return round(errorRate, 2)
+            return errorRate
 
     def resetCounters(self, nodeId):
         with self.__mutex__:
@@ -82,33 +82,6 @@ class ModbusConnection:     #Singleton
                 logging.error("[ModbusConnection] Unknown modbus connection error: " + str(e))
         return self.__modbusConnection__
 
-    def readregisters(self, node, startAddress = 1000, registersToRead = 1):
-        with self.__mutex__:
-            logging.debug("[ModbusConnection] Node " + str(node) + ": Reading " + str(registersToRead) +" registers starting with register "+ str(startAddress))
-            try:
-                self.__connectionCounter__[node] += 1
-            except:
-                self.__connectionCounter__[node] = 1
-            registers = dict()
-            if self.__modbusClient__ != None:
-                if(self.__connectSocket__()):
-                    try:
-                        modbus_response = self.__modbusClient__.read_holding_registers(startAddress, registersToRead, unit=node)
-                        if modbus_response.isError():
-                            raise Exception(str(modbus_response))
-
-                        logging.debug("[ModbusConnection] Node " + str(node) + ": Data: " + str(modbus_response.registers))
-                        for i in range(len(modbus_response.registers)):
-                            registers[startAddress + i] = modbus_response.registers[i]
-                    except Exception as e:
-                        logging.warning("[ModbusConnection] Modbus read error at node " + str(node) + ": " + str(e))
-                        self.__increaseModbusErrorCounter__(node)
-                        registers = None
-                        time.sleep(BUSRECOVERYTIME)
-                else:
-                    logging.warning("[ModbusConnection] Cannot read from node " + str(node) + ".")
-        return registers
-
     def writeregister(self, node, registerId, value):
         result = None
         with self.__mutex__:
@@ -131,32 +104,44 @@ class ModbusConnection:     #Singleton
                     logging.warning("[ModbusConnection] Cannot write to node " + str(node) + ".")
         return result
 
-    def readInputRegisters(self, node, startAddress = 1000, registersToRead = 1):
+    def readregisters(self, node, startAddress = 1000, registersToRead = 1):
+        return self.__readregisters__(node, startAddress, registersToRead, "holding")
+
+    def __readregisters__(self, node, startAddress, registersToRead, type="holding"):
         with self.__mutex__:
-            logging.debug("[ModbusConnection] Node " + str(node) + ": Reading " + str(registersToRead) +" input registers starting with register "+ str(startAddress))
+            logging.debug("[ModbusConnection] Node " + str(node) + ": Reading " + str(registersToRead) + " " + str(type) + "-registers starting with register "+ str(startAddress))
             try:
                 self.__connectionCounter__[node] += 1
             except:
                 self.__connectionCounter__[node] = 1
-            registers = dict()
             if self.__modbusClient__ != None:
                 if(self.__connectSocket__()):
+                    registers = dict()
                     try:
-                        modbus_response = self.__modbusClient__.read_input_registers(startAddress, registersToRead, unit=node)
+                        if type == "input":
+                            modbus_response = self.__modbusClient__.read_input_registers(startAddress, registersToRead, unit=node)
+                        else:
+                            modbus_response = self.__modbusClient__.read_holding_registers(startAddress, registersToRead, unit=node)
                         if modbus_response.isError():
                             raise Exception(str(modbus_response))
 
                         logging.debug("[ModbusConnection] Node " + str(node) + ": Data: " + str(modbus_response.registers))
                         for i in range(len(modbus_response.registers)):
                             registers[startAddress + i] = modbus_response.registers[i]
+
+                        return registers
+
                     except Exception as e:
                         logging.warning("[ModbusConnection] Modbus read error at node " + str(node) + ": " + str(e))
                         self.__increaseModbusErrorCounter__(node)
-                        registers = None
                         time.sleep(BUSRECOVERYTIME)
                 else:
-                    logging.warning("[ModbusConnection] Cannot read input registers from node " + str(node) + ".")
-        return registers
+                    logging.warning("[ModbusConnection] Cannot read from node " + str(node) + ".")
+            return None
+
+    def readinputregisters(self, node, startAddress = 1000, registersToRead = 1):
+        return self.__readregisters__(node, startAddress, registersToRead, "input")
+
 
 # Entry Point     
 if __name__ == "__main__":
@@ -221,7 +206,7 @@ if __name__ == "__main__":
     else:
         print(color('ERROR: Invalid node read.', fore='red', style='bright'))
 
-    # reading from broadcastr address
+    # reading from broadcast address
     register = ModbusConnection.instance().readregisters(0, 1000, 24)
     errors = ModbusConnection.instance().getModbusErrorCounter(0)
     if(register == None and errors == 1):
@@ -231,13 +216,13 @@ if __name__ == "__main__":
 
     # reading of multiple invalid registers
     ModbusConnection.instance().resetCounters(unitToTest)
-    register = ModbusConnection.instance().readregisters(unitToTest, 2000, 24)
+    register = ModbusConnection.instance().readregisters(unitToTest, 4130, 24)
     errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
     errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
     if(register == None and errors == 1 and errorrate == 1):
-        print(color('SUCCESS: Invalid register read.', fore='green', style='bright'))
+        print(color('SUCCESS: Multiple invalid register read.', fore='green', style='bright'))
     else:
-        print(color('ERROR: Invalid register read.', fore='red', style='bright'))
+        print(color('ERROR: Multiple invalid register read.', fore='red', style='bright'))
 
     # reading multiple registers multiple times
     ModbusConnection.instance().resetCounters(unitToTest)
@@ -247,6 +232,7 @@ if __name__ == "__main__":
         errors = ModbusConnection.instance().getModbusErrorCounter(unitToTest)
         errorrate =  ModbusConnection.instance().getModbusErrorRate(unitToTest)
         temp = ModbusConnection.instance().__connectionCounter__[unitToTest]
+        time.sleep(0.3)
     if(register != None and register != {} and errors == 0 and errorrate == 0 and temp == 50):
         print(color('SUCCESS: Valid multiple register read multiple times.', fore='green', style='bright'))
     else:
