@@ -4,7 +4,7 @@ import logging
 import threading
 import time
 from datetime import datetime
-import statistics
+# import statistics
 from colr import color
 import sys
 import os
@@ -13,7 +13,9 @@ PACKAGE_PARENT = '..'
 SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
 sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))
 
-BUILDNR = "1.2101.7"
+from mypvdevices.DcsConnection import DcsConnection
+
+BUILDNR = "1.2101.9"
 
 JOINTIMEOUT = 5
 SERIALLEN = 16
@@ -71,11 +73,8 @@ class ModbusDevice:
         logging.debug("new "+str(self.__devicetype__)+" created. Serial: " + str(self.__serial__))
 
     def addconnection(self, connection):
-        # if not isinstance(connection, DcsConnection):
-        #     msg="connection has to be a DcsConnection"
-        #     raise TypeError(msg)
-
-        if(connection != None):
+        # if connection != None and type(connection) is DcsConnection:
+        if connection != None:
             if(connection.addDevice(self)):
                 self.__connections__.append(connection)
                 logging.debug("[Device] ID " + str(self.__serial__) + " Added connection to device")
@@ -103,6 +102,12 @@ class ModbusDevice:
         if(self.__modbusThread__ != None and self.__modbusThread__.is_alive()):
             self.__modbusThread__.join(3)     # todo: konstante verwenden
         for connection in self.__connections__:
+            try:
+                connection.loadlogdata()
+                connection.sendlogdata()
+                logging.info("[Device] ID " + str(self.__serial__) + " logdata sent.")
+            except:
+                logging.warning("[Device] ID " + str(self.__serial__) + " cannot send logdata while stopping.")
             connection.disconnect()
     
     def getserial(self):
@@ -243,7 +248,7 @@ class ModbusDevice:
             except Exception as e:
                 logging.warning("[Device] " + self.getidentifier() + " cannot check connection to " + str(connection.getServer()) + ". " + str(e))
         if errors == 0:
-            logging.info("[Device] " + self.getidentifier() + ": everything ok.")
+            logging.debug("[Device] " + self.getidentifier() + ": DCS-Connections ok.")
 
     def __sethealthstate__(self, state):
         if state != None and state <= 3 and state >= 0:
@@ -271,10 +276,16 @@ class ModbusDevice:
             self.__readallregisters__()
             if self.__registers__ != None:
                 if len(self.__registers__) > 0:
-                    self.__syncsettings__()
-                    self.__processregisters__()
+                    try:
+                        self.__syncsettings__()
+                    except Exception as e:
+                        logging.warning("[Device] " + self.getidentifier() + ". Settingssync not successfull. " + str(e))
+                    try:
+                        self.__processregisters__()
+                    except Exception as e:
+                        logging.warning("[Device] " + self.getidentifier() + ". processing registers not successfull. " + str(e))
                 else:
-                    logging.error("[Device] " + self.getidentifier() + ". No bus communication to device.")
+                    logging.warning("[Device] " + self.getidentifier() + ". No bus communication to device.")
             else:
                 logging.debug("[Device] " + self.getidentifier() + ". No registers to process.")
             time.sleep(self.__modbusFrequency__)
@@ -288,7 +299,10 @@ class ModbusDevice:
                         if self.__registers__[self.__settingsMap__[name]["register"]] != self.__setup__[name]:
                             logging.info("[Device] ID " + str(self.__serial__) + " setting missmatch " + str(name) + " - Device: " + str(self.__registers__[self.__settingsMap__[name]["register"]]) + "; Setup: " + str(self.__setup__[name]) + ".")
                             if self.__settingsMap__[name]["forced"]:
-                                self.writeregister(self.__settingsMap__[name]["register"], self.__setup__[name])
+                                try:
+                                    self.writeregister(self.__settingsMap__[name]["register"], self.__setup__[name])
+                                except Exception as e:
+                                    logging.warning("[Device] ID " + str(self.__serial__) + " cannot write register while settings sync. Set " + str(name) + " to "+str(self.__setup__[name])+". " + str(e))
                                 logging.info("[Device] ID " + str(self.__serial__) + " forcing " + str(name) + " to "+str(self.__setup__[name])+".")
                             else:
                                 self.__setup__[name] = self.__registers__[self.__settingsMap__[name]["register"]]
@@ -320,8 +334,13 @@ class ModbusDevice:
         register = self.__readregister__(registerId)
         return register
 
+    def readregisters(self, startregisteraddress, registerstoread):
+        return self.__readregisters__(startregisteraddress, registerstoread)
+
     def __readregister__(self, registerId):
         with self.__lock__:
+            if registerId == 1234:
+                self.__registers__[1234] = 432
             try:
                 return self.__registers__[registerId]
             except Exception as e:
@@ -503,7 +522,10 @@ class ModbusDevice:
         return 0
 
     def showcommunicationerrorsrate(self):
-        logging.info("[Device] " + str(self.getidentifier()) + " - Communication error-rate: " +  str(round(self.getcommunicationerrorsrate()*100)) + "%, Errors: " +  str(self.getcommunicationerrorscounter()))
+        rate = self.getcommunicationerrorsrate()
+        if rate != None:
+            rate = round(rate*100)
+        logging.info("[Device] " + str(self.getidentifier()) + " - Communication error-rate: " +  str(rate) + "%, Errors: " +  str(self.getcommunicationerrorscounter()))
 
     def __createDataset__(self, registerId, type):
         dataset = [registerId, type]
@@ -595,7 +617,7 @@ class ModbusDevice:
 # Entry Point     
 if __name__ == "__main__":
 
-    from DcsConnection import DcsConnection
+    # from DcsConnection import DcsConnection
 
     logging.basicConfig(format='%(asctime)s %(levelname)s [%(threadName)s]: %(message)s', level=logging.INFO)
 
@@ -744,6 +766,26 @@ if __name__ == "__main__":
     else:
         print(color('ERROR: setting modbus registers to read (3).', fore='red', style='bright'))
 
+    if(device != None):
+        try:
+            register = device.readregister(1234)
+            if register == 432:
+                print(color('SUCCESS: reading register.', fore='green', style='bright'))
+            else:
+                print(color('ERROR: reading register. Value missmatch: ' + str(register), fore='red', style='bright'))
+        except Exception as e:
+            print(color('ERROR: reading register. Error: ' + str(e), fore='red', style='bright'))
+
+    if(device != None):
+        try:
+            registers = device.readregisters(1000, 10)
+            if registers[1000] == 0 and registers[1001] == 1000:
+                print(color('SUCCESS: reading registers.', fore='green', style='bright'))
+            else:
+                print(color('ERROR: reading registers. Value missmatch: ' + str(registers), fore='red', style='bright'))
+        except Exception as e:
+            print(color('ERROR: reading registers. Error: ' + str(e), fore='red', style='bright'))
+
     if(device != None):
         try:
             device.writeregister(123,33)
@@ -764,8 +806,8 @@ if __name__ == "__main__":
         except Exception as e:
             print(color('ERROR: Setting setup value with register key.', fore='red', style='bright'))
 
-    key = "testkey"
-    targetValue = 35
+    key = "fwversion"
+    targetValue = 40
     if device != None:
         try:
             device.setsetupvalue(key, targetValue)
@@ -777,6 +819,16 @@ if __name__ == "__main__":
         except Exception as e:
             print(color('ERROR: Setting setup value without register key.', fore='red', style='bright'))
 
+    key = "testkey"
+    targetValue = 33
+    if device != None:
+        try:
+            device.setsetupvalue(key, targetValue)
+            print(color('ERROR: Setting invalid setting key.', fore='red', style='bright'))
+        except Exception as e:
+            print(color('SUCCESS: Setting invalid setting key.', fore='green', style='bright'))
+            
+
     connection = DcsConnection(serial, cryptoKey, server, 50333)
     try:
         device.addconnection(connection)
